@@ -7,8 +7,7 @@ use ndarray::{rcarr1, rcarr2};
 use ndarray::{LinalgScalar, Data};
 use ndarray::linalg::general_mat_mul;
 use ndarray::linalg::general_mat_vec_mul;
-use ndarray::Si;
-use ndarray::{Ix, Ixs};
+use ndarray::{Ix, Ixs, SliceInfo, SliceOrIndex};
 
 use std::fmt;
 use std::ops::Neg;
@@ -578,16 +577,19 @@ fn scaled_add_3() {
                     vec![n, q]
                 };
                 let cslice = if n == 1 {
-                    vec![Si(0, None, s2)]
+                    vec![SliceOrIndex::from(..).step_by(s2)]
                 } else {
-                    vec![Si(0, None, s1), Si(0, None, s2)]
+                    vec![
+                        SliceOrIndex::from(..).step_by(s1),
+                        SliceOrIndex::from(..).step_by(s2),
+                    ]
                 };
 
                 let c = range_mat64(n, q).into_shape(cdim).unwrap();
 
                 {
                     let mut av = a.slice_mut(s![..;s1, ..;s2]);
-                    let c = c.slice(&cslice);
+                    let c = c.slice(SliceInfo::<_, IxDyn>::new(cslice).unwrap().as_ref());
 
                     let mut answerv = answer.slice_mut(s![..;s1, ..;s2]);
                     answerv += &(beta * &c);
